#include <QTextStream>
#include "mainwindow.h"
#include "ui_mainwindow.h"
#include "ui_setupHardware.h"

QString MainWindow::h(int len,int value)
{
    QString s;
    QTextStream out(&s);
    out.setFieldWidth(len);
    out.setIntegerBase(16);
    out.setFieldAlignment(QTextStream::AlignRight);
    out.setPadChar('0');
    out << value;
    return s;
}

QString MainWindow::doCmd(QString cmd)
{
    port->write(cmd.toAscii(),cmd.length());
    port->setTimeout(100);
    QString answer="";
    while( !answer.contains("\n") )
    {
        answer+=port->read(1);
    };
    this->statusBar()->showMessage("Last cmd: ["+cmd+"]=>["+answer+"]",3000);
    return answer;
}

void MainWindow::panTiltEnabledSwitched()
{
    bool e=uiHWSetup->panTiltEnabled->checkState();
    uiHWSetup->panServoId->setEnabled(e);
    uiHWSetup->tiltServoId->setEnabled(e);
    ui->panSlider->setEnabled(e);
    ui->tiltSlider->setEnabled(e);
}

void MainWindow::servoWheelsSwitched()
{
    bool e=uiHWSetup->servoChassis->checkState();
    uiHWSetup->leftServoId->setEnabled(e);
    uiHWSetup->rightServoId->setEnabled(e);
    uiHWSetup->leftServoOffset->setEnabled(e);
    uiHWSetup->rightServoOffset->setEnabled(e);
}

void MainWindow::getFR()
{
    servoFR=0;
    motorFR=0;
    QString s=doCmd("S40 00 00 S41 01 P\n");
    s="0x"+s.mid(6,2);
    QTextStream in(s.toAscii());
    //in.setIntegerBase(16);
    int regNum;
    in >> regNum;
    this->statusBar()->showMessage("s="+s+" regNum="+h(2,regNum),3000);
    for(int i=1; i<regNum; i++)
    {
        QString s=doCmd("S40 00 "+h(2,i)+" S41 06 P\n");
        QString d=s.mid(6,3);
        QString r="0x"+s.mid(14,2);
        QTextStream in(r.toAscii());
        in.setIntegerBase(16);
        int regFR;
        in >> regFR;

        if(d=="003") servoFR=regFR;
        if(d=="006") motorFR=regFR;

        this->statusBar()->showMessage("s="+s+" d="+d+" r="+r,3000);
    }
    this->statusBar()->showMessage("Servo Reg: ["+h(2,servoFR)+"],  Motor Reg: ["+h(2,motorFR)+"]",3000);
}

void MainWindow::btnConnect()
{
    settings->beginGroup("HWSetup");
    QString com_speed=settings->value("com_speed").toString();
    QString com_port=settings->value("com_port").toString();
    settings->endGroup();

    if(com_speed=="9600"){
        port->setBaudRate(BAUD9600);
    }else{
        port->setBaudRate(BAUD115200);
    };
    port->setPortName("COM"+com_port);

    port->open(QIODevice::ReadWrite | QIODevice::Unbuffered);

    if(!port->isOpen())
    {
        this->ui->port_label->setText("ERR:"+port->errorString());
    }else{
        conn_status="Connected";
        this->recallSettings();
        this->ui->disconnectBtn->setEnabled(true);
        this->ui->connectBtn->setEnabled(false);
        this->doCmd("L40\n");
        servoFR=0x0C;
        motorFR=0x04;
        getFR();
    };
}

void MainWindow::btnDisconnect()
{
    if(port)port->close();
    conn_status="Disconnected";
    this->recallSettings();
    this->ui->disconnectBtn->setEnabled(false);
    this->ui->connectBtn->setEnabled(true);
}

void MainWindow::recallSettings()
{
    settings->beginGroup("HWSetup");

    uiHWSetup->servoChassis->setChecked(settings->value("servo_chassis").toBool());
    uiHWSetup->leftServoId->setValue(settings->value("left_wheel_servo_id").toInt());
    uiHWSetup->rightServoId->setValue(settings->value("right_wheel_servo_id").toInt());
    uiHWSetup->leftServoOffset->setValue(settings->value("left_wheel_servo_offset").toInt());
    uiHWSetup->rightServoOffset->setValue(settings->value("right_wheel_servo_offset").toInt());

    uiHWSetup->panTiltEnabled->setChecked(settings->value("pan_tilt").toBool());
    uiHWSetup->panServoId->setValue(settings->value("pan_servo_id").toInt());
    uiHWSetup->tiltServoId->setValue(settings->value("tilt_servo_id").toInt());

    int com_port=settings->value("com_port").toInt();
    QString com_port_str=settings->value("com_port").toString();
    uiHWSetup->portSpinBox->setValue(com_port);

    QString com_speed=settings->value("com_speed").toString();
    int com_speed_num=-1;
    for(int i=0; i<uiHWSetup->speedComboBox->count(); i++){
        if(uiHWSetup->speedComboBox->itemText(i)==com_speed) com_speed_num=i;
    };
    if(com_speed_num>=0)uiHWSetup->speedComboBox->setCurrentIndex(com_speed_num);

    QString controller=settings->value("controller").toString();
    int controller_num=-1;
    for(int i=0; i<uiHWSetup->rcontrComboBox->count(); i++){
        if(uiHWSetup->rcontrComboBox->itemText(i)==controller) controller_num=i;
    };
    if(controller_num>=0)uiHWSetup->rcontrComboBox->setCurrentIndex(controller_num);

    settings->endGroup();

    this->ui->port_label->setText("COM"+com_port_str+","+com_speed+"bps,"+controller+" - "+conn_status);
}

void MainWindow::showHWSetupDialog()
{
    this->recallSettings();
    hwSetupDialog->show();
}

void MainWindow::acceptedHWSetupDialog()
{
    settings->beginGroup("HWSetup");
    settings->setValue("com_port",QVariant(uiHWSetup->portSpinBox->value()));
    settings->setValue("com_speed",QVariant(uiHWSetup->speedComboBox->currentText()));
    settings->setValue("controller",QVariant(uiHWSetup->rcontrComboBox->currentText()));
    settings->setValue("servo_chassis",QVariant(uiHWSetup->servoChassis->checkState()));
    settings->setValue("left_wheel_servo_id",QVariant(uiHWSetup->leftServoId->value()));
    settings->setValue("right_wheel_servo_id",QVariant(uiHWSetup->rightServoId->value()));
    settings->setValue("left_wheel_servo_offset",QVariant(uiHWSetup->leftServoOffset->value()));
    settings->setValue("right_wheel_servo_offset",QVariant(uiHWSetup->rightServoOffset->value()));
    settings->setValue("pan_tilt",QVariant(uiHWSetup->panTiltEnabled->checkState()));
    settings->setValue("pan_servo_id",QVariant(uiHWSetup->panServoId->value()));
    settings->setValue("tilt_servo_id",QVariant(uiHWSetup->tiltServoId->value()));
    settings->endGroup();
    this->recallSettings();
}

MainWindow::MainWindow(QWidget *parent)
    : QMainWindow(parent), ui(new Ui::MainWindow)
{
    ui->setupUi(this);

    port = new QextSerialPort("COM7");
    port->setFlowControl(FLOW_OFF);
    port->setParity(PAR_NONE);
    port->setDataBits(DATA_8);
    port->setStopBits(STOP_1);
    //set timeouts to 500 ms
    port->setTimeout(1500);

    conn_status="Disconnected";

    hwSetupDialog = new QDialog;
    uiHWSetup=new Ui::HWSetupDialog();
    uiHWSetup->setupUi(hwSetupDialog);
    uiHWSetup->speedComboBox->addItem("9600",QVariant("9600"));
    uiHWSetup->speedComboBox->addItem("115200",QVariant("115200"));

    uiHWSetup->rcontrComboBox->addItem("OR-AVR-M32-D",QVariant("M32D"));
    uiHWSetup->rcontrComboBox->addItem("OR-AVR-M128-S",QVariant("M128S"));
    uiHWSetup->rcontrComboBox->addItem("OR-AVR-M128-DS",QVariant("M128DS"));

    //QSettings::setDefaultFormat(QSettings::IniFormat);
    settings=new QSettings(QSettings::IniFormat,QSettings::UserScope,"Open Robotics","ORFA-Terminal");

    connect(this->ui->actionRobot_setup,SIGNAL(triggered()),this,SLOT(showHWSetupDialog()));
    connect(this->ui->connectBtn,SIGNAL(clicked()),this,SLOT(btnConnect()));
    connect(this->ui->disconnectBtn,SIGNAL(clicked()),this,SLOT(btnDisconnect()));
    connect(hwSetupDialog,SIGNAL(accepted()),this,SLOT(acceptedHWSetupDialog()));
    connect(uiHWSetup->servoChassis,SIGNAL(toggled(bool)),this,SLOT(servoWheelsSwitched()));
    connect(uiHWSetup->panTiltEnabled,SIGNAL(toggled(bool)),this,SLOT(panTiltEnabledSwitched()));

    this->ui->disconnectBtn->setEnabled(false);

    this->recallSettings();
}

MainWindow::~MainWindow()
{
    delete ui;
}

void MainWindow::on_pushButton_4_clicked()
{

}

void MainWindow::setChassisSpeed(int left,int right)
{
    if(uiHWSetup->servoChassis->isChecked())
    {
        QString servoReg=h(2,servoFR+1);
        QString leftServoId=this->h(2,uiHWSetup->leftServoId->value());
        QString rightServoId=this->h(2,uiHWSetup->rightServoId->value());
        QString leftSpeed=this->h(4,left);
        QString rightSpeed=this->h(4,right);
        this->doCmd("S40 "+servoReg+leftServoId+leftSpeed+rightServoId+rightSpeed+" P\n");
    }else{
        QString leftSpeed=this->h(4,abs(left-1500)/2);
        QString rightSpeed=this->h(4,abs(right-1500)/2);
        QString leftDir="00"; if(left<1500){ leftDir="01"; };
        QString rightDir="00"; if(right<1500){ rightDir="01"; };
        QString leftSpeedReg=h(2,motorFR);
        QString rightSpeedReg=h(2,motorFR+1);
        QString leftDirReg=h(2,motorFR+2);
        QString rightDirReg=h(2,motorFR+2);
        this->doCmd("S40 "+leftSpeedReg+leftSpeed+rightSpeedReg+rightSpeed+leftDirReg+leftDir+rightDirReg+rightDir+" P\n");
    }
}

void MainWindow::on_fwdButton_clicked()
{
    setChassisSpeed(2000,2000);
}

void MainWindow::on_stopButton_clicked()
{
    setChassisSpeed(1500,1500);
}

void MainWindow::on_bwdButton_clicked()
{
    setChassisSpeed(1000,1000);
}

void MainWindow::on_leftButton_clicked()
{
    setChassisSpeed(1000,2000);
}

void MainWindow::on_rightButton_clicked()
{
    setChassisSpeed(2000,1000);
}

void MainWindow::on_panSlider_valueChanged(int value)
{
    if( uiHWSetup->panTiltEnabled->isChecked() )
    {
        QString servoReg=h(2,servoFR+1);
        QString panServoId=this->h(2,uiHWSetup->panServoId->value());
        QString panPosition=this->h(4,value);
        this->doCmd("S40 "+servoReg+panServoId+panPosition+" P\n");
    }
}

void MainWindow::on_tiltSlider_valueChanged(int value)
{
    if( uiHWSetup->panTiltEnabled->isChecked() )
    {
        QString servoReg=h(2,servoFR+1);
        QString tiltServoId=this->h(2,uiHWSetup->tiltServoId->value());
        QString tiltPosition=this->h(4,value);
        this->doCmd("S40 "+servoReg+tiltServoId+tiltPosition+" P\n");
    }
}
